import os
import pandas as pd
import warnings
from typing import Optional
import logging
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import validate_and_convert_data_types, convert_to_float, convert_df_to_dict_excluding_nan
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def forward_supply_chain_map_routes(addresses: pd.DataFrame, parameters: dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Creates a map of routes based on the given route nodes addresses.

    This function takes a DataFrame of nodes addresses with their layer and pickup and delivery quantity, along with an API key, 
    and creates a map of routes using the Log-hub Supply Chain Map service. 

    Parameters:
    addresses (pd.DataFrame): A pandas DataFrame containing nodes addresses with their layer and pickup and delivery quantity.
        Each row should contain:
        - id (number): Identifier.
        - routeId (str): A unique route id. All stops with the same routeId belong to the same route.
        - name (str): Name of the route node.
        - country (str): Country code. Must have at least 1 character.
        - state (str): State code.
        - postalCode (str): Postal code.
        - city (str): City name.
        - street (str): Street name with house number.
        - layer (str): Visual settings of routes in the same layer can be adjusted together.
        - pickupQuantity (number): Quantity that is picked up at the node.
        - deliveryQuantity (number): Quantity that is delivered at the node.

    parameters (dict): A dictionary containing parameter 'streetLevel' (boolean). 

    api_key (str): The Log-hub API key for accessing the supply chain map service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                        'saveScenario' (boolean), 'overwriteScenario' (boolean), 'mergeWithExistingScenario (boolean), 'workspaceId' (str) and 'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    pd.DataFrame: A pandas DataFrame containg the nodes addresses with their layer and pickup and delivery quantity.

    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    mandatory_columns = {'routeId': 'str', 'name': 'str', 'country': 'str'}
    optional_columns = {'state': 'str', 'postalCode': 'str', 'city': 'str', 'street': 'str', 'layer': 'str'}
    optional_floats = ['id', 'pickupQuantity', 'deliveryQuantity']
    # Validate and convert data types
    addresses = validate_and_convert_data_types(addresses, mandatory_columns, 'mandatory', 'addresses')
    if not addresses is None:
        addresses = validate_and_convert_data_types(addresses, optional_columns, 'optional', 'addresses')
        if not addresses is None:
            addresses = convert_to_float(addresses, optional_floats, 'optional')
            addresses = convert_df_to_dict_excluding_nan(addresses, optional_floats)
    if addresses is None:
        return None

    url = create_url("supplychainmaproutes")
    
    headers = create_headers(api_key)
    payload = {
        "geocodingData": addresses,
        "parameters": parameters
    }
   
    payload = save_scenario_check(save_scenario, payload, "supplychainmaproutes")
    response_data = post_method(url, payload, headers, "supply chain map routes")
    if response_data is None:
        return None
    else: 
        route_result_df = pd.DataFrame(response_data['inputDataStructure'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return route_result_df
            
def forward_supply_chain_map_routes_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'MapRoutesAddresses.xlsx')
    addresses_df = pd.read_excel(data_path, sheet_name='addresses', usecols='A:K', dtype={'postalCode': str})

    parameters = {
        'streetLevel': True
    }
    save_scenario = {
        'saveScenario': True,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        "mergeWithExistingScenario": False,
        'scenarioName': 'Your scenario name'
    }
    return {'addresses': addresses_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}

def reverse_supply_chain_map_routes(coordinates: pd.DataFrame, parameters: dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Creates a map of routes based on the given route nodes coordinates.

    This function takes a DataFrame of nodes coordinates with their layer and pickup and delivery quantity, along with an API key, 
    and creates a map of routes using the Log-hub Supply Chain Map service. 

    Parameters:
    coordinates (pd.DataFrame): A pandas DataFrame containing coordinates with their layer and pickup and delivery quantity.
        Each row should contain:
        - id (number): Identifier.
        - routeId (str): A unique route id. All stops with the same routeId belong to the same route.
        - name (str): Name of the route node.
        - latitude (number): Latitude of the route node.
        - longitude (number): Longitude of the route node.
        - layer (str): Visual settings of routes in the same layer can be adjusted together.
        - pickupQuantity (number): Quantity that is picked up at the node.
        - deliveryQuantity (number): Quantity that is delivered at the node.
    
    parameters (dict): A dictionary containing parameter 'streetLevel' (boolean). 

    api_key (str): The Log-hub API key for accessing supply chain map service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                        'saveScenario' (boolean), 'overwriteScenario' (boolean), 'mergeWithExistingScenario (boolean), 'workspaceId' (str) and 'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    pd.DataFrame: A pandas DataFrame containg the route nodes coordinates. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    mandatory_columns = {'routeId': 'str', 'name': 'str', 'latitude':'float', 'longitude': 'float'}
    optional_columns = {'layer': 'str'}
    optional_floats = ['id', 'pickupQuantity', 'deliveryQuantity']
    # Validate and convert data types
    coordinates = validate_and_convert_data_types(coordinates, mandatory_columns, 'mandatory', 'coordinates')
    if not coordinates is None:
        coordinates = validate_and_convert_data_types(coordinates, optional_columns, 'optional', 'coordinates')
        if not coordinates is None:
            coordinates = convert_to_float(coordinates, optional_floats, 'optional')
            coordinates = convert_df_to_dict_excluding_nan(coordinates, optional_floats)
    if coordinates is None:
        return None
    url = create_url("reversesupplychainmaproutes")
    
    headers = create_headers(api_key)
    payload = {
        "routeLatLon": coordinates,
        "parameters": parameters
    }
    
    payload = save_scenario_check(save_scenario, payload, "reversesupplychainmaproutes")
    response_data = post_method(url, payload, headers, "reverse supply chain map routes")
    if response_data is None:
        return None
    else:
        route_result_df = pd.DataFrame(response_data['inputDataStructure'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return route_result_df
           
def reverse_supply_chain_map_routes_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'MapRoutesReverse.xlsx')
    coordinates_df = pd.read_excel(data_path, sheet_name='coordinates', usecols='A:H')

    parameters = {
        'streetLevel': True
    }
    save_scenario = {
        'saveScenario': True,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        "mergeWithExistingScenario": False,
        'scenarioName': 'Your scenario name'
    }
    return {'coordinates': coordinates_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}


import os
import pandas as pd
import logging
from typing import Optional, Dict
import warnings
logging.basicConfig(level=logging.INFO)
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import validate_and_convert_data_types
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def forward_isochrone(addresses: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Perform forward isochrone calculation on a list of addresses.

    This function takes a DataFrame of addresses, a dictionary of parameters and an API key, and calculates an
    isochrone using the Log-hub forward isochrone service. 

    Parameters:
    addresses (pd.DataFrame): A pandas DataFrame containing the addresses to be geocoded.
        Required columns and their types:
        - name (str): The name of the isochrone.
        - country (str): Country code. Must have at least 1 character.
        - state (str): State code.
        - postalCode (str): Postal code.
        - city (str): City name.
        - street (str): Street name with house number.
    
    parameters (dict): A dictionary containing parameters for isochrone calculation. 
        - isochroneType: "road" or "beeline"
        - distanceUnit: "km" or "mi"
        - travelMode: "driving-car" or "driving-hgv", used if isochroneType is "road"
        - maxTravelTime: a number between 1 and 300, used if isochroneType is "road"
        - layer: a number between 1 and 10, used if isochroneType is "road"
        - maxDistance: a number between 1 and 3000, used if isochroneType is "beeline"
        - layerBeeline: a number between 1 and 25, used if isochroneType is "beeline"

    api_key (str): The Log-hub API key for accessing the isochrone service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    pd.DataFrame: A pandas DataFrame containing the original address information along 
                  with the geocoded results, cordinates, and information about area and population.
                  Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    mandatory_columns = {'name': 'str', 'country': 'str'}
    optional_columns = {'state': 'str', 'postalCode': 'str', 'city': 'str', 'street': 'str'}

    # Validate and convert data types
    addresses = validate_and_convert_data_types(addresses, mandatory_columns, 'mandatory', 'addresses')
    if not addresses is None:
        addresses = validate_and_convert_data_types(addresses, optional_columns, 'optional', 'addresses')

    if addresses is None:
        return None
    
    url = create_url("isochrone")
    
    headers = create_headers(api_key)

    payload = {
        'geocodingData': addresses.to_dict(orient='records'),
        'parameters': parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "isochrone")
    if response_data is None:
        return None
    else:
        geocoded_data_df = pd.DataFrame(response_data['geocodingResult'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return geocoded_data_df

def forward_isochrone_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'IsochroneSampleDataAddresses.xlsx')
    addresses_df = pd.read_excel(data_path, sheet_name='addresses', usecols='A:G').fillna("")
    parameters = {
        'isochroneType': 'road',
        'distanceUnit': 'km',
        'travelMode': 'car',
        'maxTravelTime': 40,
        'layers': 5,
        'maxDistance': 3000,
        'layersBeeline': 5
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'addresses': addresses_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}

def reverse_isochrone(geocodes: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Perform reverse isochrone calculation on a list of latitude and longitude coordinates.

    This function takes a DataFrame of geocodes (latitude and longitude), parameters and an API key, 
    and calculates isochrones using the Log-hub reverse isochrone service. 

    Parameters:
    geocodes (pd.DataFrame): A pandas DataFrame containing the geocodes to be reverse geocoded.
        Required columns and their types:
        - name (str): The name of the isochrone.
        - latitude (float): Latitude of the location.
        - longitude (float): Longitude of the location.

    parameters (dict): A dictionary containing parameters for isochrone calculation. 
        - isochroneType: "road" or "beeline"
        - distanceUnit: "km" or "mi"
        - travelMode: "driving-car" or "driving-hgv", used if isochroneType is "road"
        - maxTravelTime: A number between 1 and 300, used if isochroneType is "road"
        - layer: A number between 1 and 10, used if isochroneType is "road"
        - maxDistance: A number between 1 and 3000, used if isochroneType is "beeline"
        - layerBeeline: A number between 1 and 25, used if isochroneType is "beeline"

    api_key (str): The Log-hub API key for accessing the reverse isochrone service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    pd.DataFrame: A pandas DataFrame containing the original geocode information along 
                  with the reverse geocoded address results, area and population information. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    geocodes_columns = {'name': 'str', 'latitude': 'float', 'longitude': 'float'}

    # Validate and convert data types
    geocodes = validate_and_convert_data_types(geocodes, geocodes_columns, 'mandatory', 'coordinates')
    if geocodes is None:
        return None
    
    url = create_url("reverseisochrone")
    headers = create_headers(api_key)
    payload = {
        "geocodingData": geocodes.to_dict(orient='records'),
        "parameters": parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "reverse isochrone")
    if response_data is None:
        return None
    else:
        result_df = pd.DataFrame(response_data['geocodingResult'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return result_df

def reverse_isochrone_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'IsochroneSampleDataReverse.xlsx')
    geocodes_df = pd.read_excel(data_path, sheet_name='coordinates', usecols='A:D')

    parameters = {
        'isochroneType': 'road',
        'distanceUnit': 'km',
        'travelMode': 'car',
        'maxTravelTime': 40,
        'layers': 5,
        'maxDistance': 3000,
        'layersBeeline': 5
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'coordinates': geocodes_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}


import logging
import os
logging.basicConfig(level=logging.INFO)
import webbrowser
from IPython.display import display, Javascript, HTML
import ipywidgets as widgets
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

def get_app_name(url):

    app_name = url.split("/")[-1]

    return app_name


def save_scenario_check(save_scenario_data: dict, payload: dict, application_name = "") -> dict:
    """
    Updates the payload with the information about saving a scenario. If any of the parameters for saving a scenario was not passed, it will be filled with the default value.

    Parameters:
    save_scenario_data (dict): A dictionary with the parameters for saving a scenario:
        - saveScenario (boolean): False by default
        - overwriteScenario (boolean): False by default
        - mergeWithExistingScenario (boolean): False by default
        - workspaceId (str): Id of an existing workspace
        - scenarioName (str): Scenario name
    payload (dict): A dictionaty with the data that should be calculated
    application_name (str): Application name

    Returns:
    dict: A dictionary containing input data and the information about saving a scenario.
    """
    save_scenario_parameters = ['saveScenario', 'overwriteScenario','workspaceId', 'scenarioName']
    save_scenario_default_values = {
        'saveScenario' : False,
        'overwriteScenario': False,
        'workspaceId': "",
        'scenarioName': ""
    }
    if 'supplychainmap' in application_name:
        save_scenario_parameters.insert(2, 'mergeWithExistingScenario')
        save_scenario_default_values.update({'mergeWithExistingScenario': False})
        save_scenario_default_values['saveScenario'] = True
    payload['saveScenarioParameters']  = {}
    for par in save_scenario_parameters:
        payload['saveScenarioParameters'].update({par: save_scenario_data.get(par, save_scenario_default_values[par])})

    return payload

def get_entities_link(data, scenario_name):

    DEFAULT_LOG_HUB_API_SERVER = "https://production.supply-chain-apps.log-hub.com"
    LOG_HUB_API_SERVER = os.getenv('LOG_HUB_API_SERVER', DEFAULT_LOG_HUB_API_SERVER)

    entities_id = {}

    for line in data:
        if line['name'] == scenario_name and line['type'] == "MAP":
            entities_id['map'] = f"{LOG_HUB_API_SERVER}/sca/platform/workspaces/" + line['workspaceId'] + "/maps/" + line['_id']
        if line['name'] == scenario_name and line['type'] == "DASHBOARD":
            entities_id['dashboard'] = f"{LOG_HUB_API_SERVER}/sca/platform/workspaces/" + line['workspaceId'] + "/dashboards/" + line['_id']
        if line['name'] == scenario_name + " Input" and line['type'] == "DATASET":
            entities_id['inputDataset'] = f"{LOG_HUB_API_SERVER}/sca/platform/workspaces/" + line['workspaceId'] + "/datasets/" + line['_id']
        if line['name'] == scenario_name + " Output" and line['type'] == "DATASET":
            entities_id['outputDataset'] = f"{LOG_HUB_API_SERVER}/sca/platform/workspaces/" + line['workspaceId'] + "/datasets/" + line['_id']
    
    if  entities_id == {}:
        return None
    else:
        return entities_id

 
def create_button(links, texts):
    """
    Creates a clean, minimal button for Jupyter Notebook output that links to the specified URL.
 
    Parameters:
      link (str): The URL to open when the button is clicked.
      text (str): The text displayed on the button.
    """
    buttons_html = ""
    for link, text in zip(links, texts):
        button_html = f'''
            <a href="{link}" target="_blank" style="text-decoration: none; display: inline-block; margin-right: 10px;">
            <button style="
                font-family: 'Open Sans', sans-serif;
                font-weight: 600;
                background-color: #F7F8FA;
                color: #111;
                border: 1px solid #E5E7EB;
                padding: 10px 20px;
                font-size: 14px;
                letter-spacing: 0.5px;
                border-radius: 6px;
                transition: background-color 0.2s ease, box-shadow 0.2s ease;
                margin-bottom: 10px;
            "
            onmouseover="this.style.backgroundColor='#E5E7EB'; this.style.boxShadow='0 2px 4px rgba(0,0,0,0.1)';"
            onmouseout="this.style.backgroundColor='#F7F8FA'; this.style.boxShadow='none';">
            {text}
            </button>
            </a>
            '''
        buttons_html += button_html

    display(HTML(buttons_html))



import os
import logging
import pandas as pd
import warnings
from typing import Optional, Dict, Tuple
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import convert_timestamps, validate_and_convert_data_types, convert_to_float, convert_df_to_dict_excluding_nan
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def forward_transport_optimization(locations: pd.DataFrame, vehicle_types: pd.DataFrame, shipments: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]]:
    """
    Perform transport optimization based on locations, vehicle types, and shipments.

    This function takes three DataFrames representing locations, vehicle types, and shipments, 
    along with a parameters dictionary, an API key and parameters for saving the scenario. It performs transport optimization using the Log-hub service.

    Parameters:
    locations (pd.DataFrame): DataFrame containing locations information.
        Columns:
        - name (str): Name of the location.
        - country (str): Country code.
        - state (str): State code.
        - postalCode (str): Postal code.
        - city (str): City name.
        - street (str): Street name with house number.

    vehicle_types (pd.DataFrame): DataFrame containing vehicle type information.
        Columns:
        - type (str): Vehicle Type.
        - availableVehicles (number): The number of available vehicles per vehicle type.
        - startLocation (str): Start location name of the vehicle type.
        - endLocation (str): End location name of the vehicle type.
        - averageSpeed (number): Average speed.
        - maxRouteStops (number): The maximum number of stops per vehicle.
        - maxRouteLength (number): The maximum length per route. 
        - maxRouteDuration (number): The maximum route duration.
        - maxCapacityWeight (number): The maximum weight that can be loaded by a vehicle of the corresponding vehicle type. 
        - maxCapacityVolume (number): The maximum volume that can be loaded by a vehicle of the corresponding vehicle type. 
        - maxCapacityPallets (number):  The maximum number of pallets that can be loaded by a vehicle of the corresponding vehicle type.
        - fixedCosts (number): The fixed costs per vehicle in case of usage. 
        - costsPerStop (number): The costs per vehicle stop.
        - costsPerDistanceUnit (number): The costs per distance unit. 

     shipments (pd.DataFrame): DataFrame containing shipments information.
        Columns:
        - name (str): The name of the shipment. 
        - senderId (str): Sender id of the shipment.
        - senderStopDuration (float): Service time needed on the pickup location of the shipment.
        - earliestPickupTime (str): Start of the pickup time window per shipment.
        - latestPickupTime (str): End of the pickup time window per shipment.
        - recipientId (str): Recipient id of the shipment.
        - recipientStopDuration (float): Service time needed on the delivery location of the shipment.
        - earliestDeliveryTime (str): Start of the delivery time window per shipment.
        - latestDeliveryTime (str): End of the delivery time window per shipment.
        - weight (number): The weight of the shipment. 
        - volume (number): The volume of the shipment.
        - pallets (number): The number of pallets of the shipment.
        - externalCosts (number): External costs for shipment.
        - vehicleType (str): All valid vehicle types for the order separated by semicolon.   

    parameters (Dict): Dictionary containing parameters durationUnit and distanceUnit.

    api_key (str): Log-hub API key for accessing the transport optimization service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                        'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                        'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]: Five pandas DataFrames containing transport overview, transport details, dropped shipments, input map routes with addresses information and input map routes with coordinates information. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['dashboard'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📊 Open Dashboard", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    # Define expected columns and data types for each DataFrame
    locations_mandatory_columns = {
        'country': 'str', 'name': 'str'
    }
    locations_optional_columns = {
        'state': 'str', 'postalCode': 'str', 'city': 'str', 'street': 'str'
    }

    vehicle_types_mandatory_columns = {
        'type': 'str', 'availableVehicles': 'float',  'startLocation': 'str', 'endLocation': 'str', 'averageSpeed': 'float', 'maxRouteStops': 'float', 'maxRouteLength': 'float', 'maxRouteDuration': 'float', 'maxCapacityWeight': 'float', 'maxCapacityVolume': 'float', 'maxCapacityPallets': 'float', 'fixedCosts': 'float', 'costsPerStop': 'float', 'costsPerDistanceUnit': 'float'
    }   
    shipments_mandatory_columns = {
        'name': 'str', 'senderId': 'str', 'senderStopDuration': 'float', 'earliestPickupTime': 'str', 'latestPickupTime': 'str', 'recipientId': 'str', 'recipientStopDuration': 'float', 'earliestDeliveryTime': 'str',  'latestDeliveryTime': 'str', 'weight': 'float', 'externalCosts': 'float'
    }
    shipemnts_optional_columns = {
        'vehicleType': 'str'
    }
    shipments_optional_floats = ['volume', 'pallets']

    # Perform validation and conversion for each DataFrame
    locations = validate_and_convert_data_types(locations, locations_mandatory_columns, 'mandatory', 'locations')
    if not locations is None:
        locations = validate_and_convert_data_types(locations, locations_optional_columns, 'optional', 'locations')
    vehicle_types = validate_and_convert_data_types(vehicle_types, vehicle_types_mandatory_columns, 'mandatory', 'vehicle types')
    shipments = validate_and_convert_data_types(shipments, shipments_mandatory_columns, 'mandatory', 'shipments')
    if not shipments is None:
        shipments = validate_and_convert_data_types(shipments, shipemnts_optional_columns, 'optional', 'shipments')
        if not shipments is None:
            shipments = convert_to_float(shipments, shipments_optional_floats, 'optional')
            shipments = convert_timestamps(shipments)
            shipments = convert_df_to_dict_excluding_nan(shipments, shipments_optional_floats)

    # Exit if any DataFrame validation failed
    if any(df is None for df in [locations, vehicle_types, shipments]):
        return None

    url = create_url("transportoptimization")
    
    headers = create_headers(api_key)

    payload = {
        "locations": locations.to_dict(orient='records'),
        "vehicleTypes": vehicle_types.to_dict(orient='records'),
        "shipments": shipments,
        "parameters": parameters
    }
    payload = save_scenario_check(save_scenario, payload)
    response_data = post_method(url, payload, headers, "transport optimization")
    if response_data is None:
        return None
    else:
        transport_overview_df = pd.DataFrame(response_data['transportOverview'])
        transport_details_df = pd.DataFrame(response_data['transportDetails'])
        dropped_shipments_df = pd.DataFrame(response_data['droppedShipments'])
        input_map_routes_df = pd.DataFrame(response_data['inputMapRoutes'])
        input_map_routes_geocodes_df = pd.DataFrame(response_data['inputMapRoutesGeocodes'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return transport_overview_df, transport_details_df, dropped_shipments_df, input_map_routes_df, input_map_routes_geocodes_df

def forward_transport_optimization_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'transportAddresses.xlsx')
    locations_df = pd.read_excel(data_path, sheet_name='locations', usecols='A:G').fillna("")
    vehicle_types_df = pd.read_excel(data_path, sheet_name='vehicleTypes', usecols='A:O').fillna("")
    shipments_df = pd.read_excel(data_path, sheet_name='shipments', usecols='A:O').fillna("")

    parameters = {
        "durationUnit": "min",
        "distanceUnit": "km"
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'locations': locations_df, 'vehicleTypes': vehicle_types_df, 'shipments': shipments_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}

def reverse_transport_optimization(locations: pd.DataFrame, vehicle_types: pd.DataFrame, shipments: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]]:
    """
    Perform transport optimization based on locations, vehicle types, and shipments.

    This function takes three DataFrames representing locations, vehicle types, and shipments, 
    along with a parameters dictionary, an API key and parameters for saving the scenario. It performs transport optimization using the Log-hub service.

    Parameters:
    locations (pd.DataFrame): DataFrame containing locations information.
        Columns:
        - name (str): Name of the location.
        - latitude (number): Latitude of the location.
        - longitude (number): Longitude of the location.

    vehicle_types (pd.DataFrame): DataFrame containing vehicle type information.
        Columns:
        - type (str): Vehicle Type.
        - availableVehicles (number): The number of available vehicles per vehicle type.
        - startLocation (str): Start location name of the vehicle type.
        - endLocation (str): End location name of the vehicle type.
        - averageSpeed (number): Average speed.
        - maxRouteStops (number): The maximum number of stops per vehicle.
        - maxRouteLength (number): The maximum length per route. 
        - maxRouteDuration (number): The maximum route duration.
        - maxCapacityWeight (number): The maximum weight that can be loaded by a vehicle of the corresponding vehicle type. 
        - maxCapacityVolume (number): The maximum volume that can be loaded by a vehicle of the corresponding vehicle type. 
        - maxCapacityPallets (number):  The maximum number of pallets that can be loaded by a vehicle of the corresponding vehicle type.
        - fixedCosts (number): The fixed costs per vehicle in case of usage. 
        - costsPerStop (number): The costs per vehicle stop.
        - costsPerDistanceUnit (number): The costs per distance unit. 

     shipments (pd.DataFrame): DataFrame containing shipments information.
        Columns:
        - name (str): The name of the order. 
        - senderId (str): Sender id of the shipment.
        - senderStopDuration (float): Service time needed on the pickup location of the shipment.
        - earliestPickupTime (str): Start of the pickup time window per shipment.
        - latestPickupTime (str): End of the pickup time window per shipment.
        - recipientId (str): Recipient id of the shipment.
        - recipientStopDuration (float): Service time needed on the delivery location of the shipment.
        - earliestDeliveryTime (str): Start of the delivery time window per shipment.
        - latestDeliveryTime (str): End of the delivery time window per shipment.
        - weight (number): The weight of the shipment. 
        - volume (number): The volume of the shipment.
        - pallets (number): The number of pallets of the shipment.
        - externalCosts (number): External costs for shipment.
        - vehicleType (str): All valid vehicle types for the order separated by semicolon.   

    parameters (Dict): Dictionary containing parameters durationUnit and distanceUnit.

    api_key (str): Log-hub API key for accessing the reverse transport optimization service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                        'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                        'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]: Four pandas DataFrames containing transport overview, trasnport details, dropped shipments and input map routes with coordinates information. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['dashboard'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📊 Open Dashboard", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    # Define expected columns and data types for each DataFrame
    locations_mandatory_columns = {
        'name': 'str', 'latitude': 'float', 'longitude': 'float'
    }

    vehicle_types_mandatory_columns = {
        'type': 'str', 'availableVehicles': 'float',  'startLocation': 'str', 'endLocation': 'str', 'averageSpeed': 'float', 'maxRouteStops': 'float', 'maxRouteLength': 'float', 'maxRouteDuration': 'float', 'maxCapacityWeight': 'float', 'maxCapacityVolume': 'float', 'maxCapacityPallets': 'float', 'fixedCosts': 'float', 'costsPerStop': 'float', 'costsPerDistanceUnit': 'float'
    }   
    shipments_mandatory_columns = {
        'name': 'str', 'senderId': 'str', 'senderStopDuration': 'float', 'earliestPickupTime': 'str', 'latestPickupTime': 'str', 'recipientId': 'str', 'recipientStopDuration': 'float', 'earliestDeliveryTime': 'str',  'latestDeliveryTime': 'str', 'weight': 'float', 'externalCosts': 'float'
    }
    shipemnts_optional_columns = {
        'vehicleType': 'str'
    }
    shipments_optional_floats = ['volume', 'pallets']

    # Perform validation and conversion for each DataFrame
    locations = validate_and_convert_data_types(locations, locations_mandatory_columns, 'mandatory', 'locations')
    vehicle_types = validate_and_convert_data_types(vehicle_types, vehicle_types_mandatory_columns, 'mandatory', 'vehicle types')
    shipments = validate_and_convert_data_types(shipments, shipments_mandatory_columns, 'mandatory', 'shipments')
    if not shipments is None:
        shipments = validate_and_convert_data_types(shipments, shipemnts_optional_columns, 'optional', 'shipments')
        if not shipments is None:
            shipments = convert_to_float(shipments, shipments_optional_floats, 'optional')
            shipments = convert_timestamps(shipments)
            shipments = convert_df_to_dict_excluding_nan(shipments, shipments_optional_floats)


    # Exit if any DataFrame validation failed
    if any(df is None for df in [locations, vehicle_types, shipments]):
        return None

    url = create_url("reversetransportoptimization")
    
    headers = create_headers(api_key)

    payload = {
        "locations": locations.to_dict(orient='records'),
        "vehicleTypes": vehicle_types.to_dict(orient='records'),
        "shipments": shipments,
        "parameters": parameters
    }
    payload = save_scenario_check(save_scenario, payload)
    response_data = post_method(url, payload, headers, "reverse transport optimization")
    if response_data is None:
        return None
    else:
        transport_overview_df = pd.DataFrame(response_data['transportOverview'])
        transport_details_df = pd.DataFrame(response_data['transportDetails'])
        dropped_shipments_df = pd.DataFrame(response_data['droppedShipments'])
        input_map_routes_geocodes_df = pd.DataFrame(response_data['inputMapRoutesGeocodes'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return transport_overview_df, transport_details_df, dropped_shipments_df, input_map_routes_geocodes_df

def reverse_transport_optimization_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'TransportSampleDataReverse.xlsx')
    locations_df = pd.read_excel(data_path, sheet_name='locations', usecols='A:D')
    vehicle_types_df = pd.read_excel(data_path, sheet_name='vehicleTypes', usecols='A:O')
    shipments_df= pd.read_excel(data_path, sheet_name='shipments', usecols='A:O')

    parameters = {
        "distanceUnit": "km",
        "durationUnit": "min"
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'locations': locations_df, 'vehicleTypes': vehicle_types_df, 'shipments': shipments_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}


from setuptools import setup, find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='pyloghub',
    version='0.1.8',
    packages=find_packages(),
    description='The `pyloghub` package provides convinient access to various Log-hub API services for Supply Chain Visualization, Network Design Optimization, and Transport Optimization as well as access to the Log-hub platform data.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/log-hub/log-hub-python',
    author='Log-hub AG',
    author_email='support@log-hub.com',
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=required,
    package_data={'pyloghub': ['sample_data/*.xlsx']},
    include_package_data=True,
)



import os
import pandas as pd
import logging
from typing import Optional
import warnings
logging.basicConfig(level=logging.INFO)
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import convert_dates, validate_and_convert_data_types, convert_to_float, convert_df_to_dict_excluding_nan
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def forward_freight_shipment_emissions_air(iata_codes: pd.DataFrame, parameters: dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Perform forward freight emissions by air on a list of IATA Codes.

    This function takes a DataFrame of IATA Codes and an API key, and performs forward
    freight emissions by air using the Log-hub service. 

    Parameters:
    iata_codes (pd.DataFrame): A pandas DataFrame containing the IATA Codes for calculating a CO2 footprint based on your shipments or route legs defined as separate shipments.
        Required columns and their types:
        - shipmentId (str): Unique identifier for each shipment.
        - shipmentDate (str): Dates related to the shipment (Format: YYYY-MM-DD)
        - fromIataCode (str): IATA Code for the shipment's start location.
        - toIataCode (str): IATA Code for the shipment's end location.
        - isRefrigirated (str): A YES/NO option that specifies whether the content being transferred through the shipments is refrigirated or not. If not specified will be taken as NO.
        - distance (number): Distance between sender and recipient. If not provided, the distance will be calculated.
        - weight (number): The weight of the shipment.

    parameters (dict): A dictionary containing parameters:
        - weightUnit: enum "kilograms" or "LBS"
        - planeType: enum "average", "freighter" or "bellyFreight"
        - refrigerantFactor: a number between 0 and 100 

    api_key (str): The Log-hub API key for accessing the freight emissions service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).
    
    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Optional[pd.DataFrame]: A pandas DataFrame contains the original shipment information along 
                  with the calculated CO2 emissions. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['dashboard'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📊 Open Dashboard", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    iata_codes_mandatory_columns = {'shipmentId': 'str', 'shipmentDate': 'str', 'fromIataCode': 'str', 'toIataCode': 'str', 'weight': 'float'}
    iata_codes_optional_columns = {'isRefrigirated': 'str'}
    iata_codes_optional_floats = ['distance']

    # Validate and convert data types
    iata_codes = validate_and_convert_data_types(iata_codes, iata_codes_mandatory_columns, 'mandatory', 'iata codes')
    if not iata_codes is None:
        iata_codes = validate_and_convert_data_types(iata_codes, iata_codes_optional_columns, 'optional', 'iata codes')
        if not iata_codes is None:
            iata_codes = convert_dates(iata_codes, ['shipmentDate'])
            iata_codes = convert_to_float(iata_codes, iata_codes_optional_floats, 'optional')
            iata_codes = convert_df_to_dict_excluding_nan(iata_codes, iata_codes_optional_floats)
    if iata_codes is None:
        return None
    
    url = create_url("co2emissionsair")
    
    headers = create_headers(api_key)

    payload = {
        'freightShipmentEmissionsByAir': iata_codes,
        'parameters': parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "co2 emissions air")
    if response_data is None:
        return None
    else:
        freight_emissions_df = pd.DataFrame(response_data['freightShipmentEmissionOutputAir'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return freight_emissions_df

def forward_freight_shipment_emissions_air_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'CO2Air.xlsx')
    iata_codes_df = pd.read_excel(data_path, sheet_name='air', usecols='A:H').fillna("")
    parameters = {
        "planeType": "average",
        "weightUnit": "kilograms",
        "refrigerantFactor": 12
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'iataCodes': iata_codes_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}

def reverse_freight_shipment_emissions_air(coordinates: pd.DataFrame, parameters: dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Perform reverse freight emissions by air on a list of from and to coordinates.

    This function takes a DataFrame of from and to locations coordinates and an API key, and performs reverse
    freight emissions by air using the Log-hub service. 

    Parameters:
    coordinates (pd.DataFrame): A pandas DataFrame containing the from and to locations coordinates for calculating a CO2 footprint based on your shipments or route legs defined as separate shipments.
        Required columns and their types:
        - shipmentId (str): Unique identifier for each shipment.
        - shipmentDate (str): Dates related to the shipment (Format: YYYY-MM-DD)
        - fromLatitude (number): The latitude of the coordinate from which a shipment should be sent.
        - fromLongitude(number): The longitude of the coordinate from which a shipment should be sent.
        - toLatitude (number): The latitude of the coordinate to which a shipment should arrive at.
        - toLongitude (numebr): The longitude of the coordinate to which a shipment should arrive at.
        - isRefrigirated (str): A YES/NO option that specifies whether the content being transferred through the shipments is refrigirated or not. If not specified will be taken as NO.
        - distance (number): Distance between sender and recipient. If not provided, the distance will be calculated.
        - weight (number): The weight of the shipment.

    parameters (dict): A dictionary containing parameters:
        - weightUnit: enum "kilograms" or "LBS"
        - planeType: enum "average", "freighter" or "bellyFreight"
        - refrigerantFactor: a number between 0 and 100 

    api_key (str): The Log-hub API key for accessing the freight emissions service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).
    
    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Optional[pd.DataFrame]: A pandas DataFrame contains the original shipment information along 
                  with the calculated CO2 emissions. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['dashboard'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📊 Open Dashboard", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    coordinates_mandatory_columns = {'shipmentId': 'str', 'shipmentDate': 'str', 'fromLatitude': 'float', 'fromLongitude': 'float', 'toLatitude': 'float', 'toLongitude': 'float', 'weight': 'float'}
    coordinates_optional_columns = {'isRefrigirated': 'str'}
    coordinates_optional_floats = ['distance']

    # Validate and convert data types
    coordinates = validate_and_convert_data_types(coordinates, coordinates_mandatory_columns, 'mandatory', 'coordinates')
    if not coordinates is None:
        coordinates = validate_and_convert_data_types(coordinates, coordinates_optional_columns, 'optional', 'coordinates')
        if not coordinates is None:
            coordinates = convert_dates(coordinates, ['shipmentDate'])
            coordinates = convert_to_float(coordinates, coordinates_optional_floats, 'optional')
            coordinates = convert_df_to_dict_excluding_nan(coordinates, coordinates_optional_floats)
    if coordinates is None:
        return None
    
    url = create_url("reverseco2emissionsair")
    
    headers = create_headers(api_key)

    payload = {
        'freightShipmentEmissionsByAir': coordinates,
        'parameters': parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "reverse co2 emissions air")
    if response_data is None:
        return None
    else:
        freight_emissions_df = pd.DataFrame(response_data['freightShipmentEmissionOutputReverseAir'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return freight_emissions_df

def reverse_freight_shipment_emissions_air_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'CO2AirReverse.xlsx')
    coordinates_df = pd.read_excel(data_path, sheet_name='air', usecols='A:J').fillna("")
    parameters = {
        "planeType": "average",
        "weightUnit": "kilograms",
        "refrigerantFactor": 12
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'coordinates': coordinates_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}


import os
import pandas as pd
import warnings
from typing import Optional
import logging
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import validate_and_convert_data_types
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def reverse_supply_chain_map_polyline(polyline: pd.DataFrame, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[pd.DataFrame]:
    """
    Creates a polyline on the map.

    This function takes a DataFrame of polyline with its latitude and longitude pairs and layer, along with an API key, 
    and creates a polyline using the Log-hub Supply Chain Map service. 

    Parameters:
    coordinates (pd.DataFrame): A pandas DataFrame containing coordinates with their layer, quantity and descriptions.
        Each row should contain:
        - id (number): Identifier.
        - polyline (str): Polyline name.
        - latitude (number): Latitude of the polyline.
        - longitude (number): Longitude of the polyline.
        - layer (str): Layer that polyline is going to be assigned to.
        
    api_key (str): The Log-hub API key for accessing the supply chain map service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                        'saveScenario' (boolean), 'overwriteScenario' (boolean), 'mergeWithExistingScenario (boolean), 'workspaceId' (str) and 'scenarioName' (str).
    
    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    pd.DataFrame: A pandas DataFrame containg the polylines. Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    mandatory_columns = {
            'polyline': 'str', 'latitude':'float', 'longitude': 'float'
        }
    optional_columns = {'id': 'float', 'layer': 'str'}

    # Validate and convert data types
    polyline = validate_and_convert_data_types(polyline, mandatory_columns, 'mandatory', 'polyline')
    if not polyline is None:
        polyline = validate_and_convert_data_types(polyline, optional_columns, 'optional', 'polyline')
    if polyline is None:
        return None

    url = create_url("reversesupplychainmappolyline")
    
    headers = create_headers(api_key)
    payload = {
        "polylineData": polyline.to_dict(orient='records'),
    }
    
    payload = save_scenario_check(save_scenario, payload, "reversesupplychainmappolyline")
    
    response_data = post_method(url, payload, headers, "supplychain map polyline")
    if response_data is None:
        return None
    else:
        polyline_df = pd.DataFrame(response_data['polylineData'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return polyline_df

def reverse_supply_chain_map_polyline_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'MapPolyline.xlsx')
    polyline_df = pd.read_excel(data_path, sheet_name='polyline', usecols='A:E')

    save_scenario = {
        'saveScenario': True,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        "mergeWithExistingScenario": False,
        'scenarioName': 'Your scenario name'
    }
    return {'polyline': polyline_df, 'saveScenarioParameters': save_scenario}


import os
import pandas as pd
import logging
from typing import Optional, Dict, Tuple
import warnings
logging.basicConfig(level=logging.INFO)
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'pyloghub')))
from save_to_platform import save_scenario_check, create_button
from input_data_validation import validate_and_convert_data_types
from sending_requests import post_method, create_headers, create_url, get_workspace_entities

def forward_isochrone_plus(addresses: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[Tuple[pd.DataFrame, pd.DataFrame]]:
    """
    Perform forward isochrone plus calculation on a list of addresses.

    This function takes a DataFrame of addresses, a dictionary of parameters and an API key, and calculates an
    isochrone using the Log-hub forward isochrone plus service. 

    Parameters:
    addresses (pd.DataFrame): A pandas DataFrame containing the addresses to be geocoded.
        Required columns and their types:
        - name (str): The name of the isochrone.
        - country (str): Country code. Must have at least 1 character.
        - state (str): State code.
        - postalCode (str): Postal code.
        - city (str): City name.
        - street (str): Street name with house number.
    
    parameters (dict): A dictionary containing parameters for isochrone calculation. 
        - level (str): A number between 1 and 5, sent as a string, and representing administration level.
        - maxTravelTime1 (number): A number between 1 and 1000. The maximum duration for Administration Level 1.
        - maxTravelTime2 (number): A number between 1 and 600. The maximum duration for Administration Level 2.
        - maxTravelTime3 (number): A number between 1 and 600. The maximum duration for Administration Level 3.
        - maxTravelTime4 (number): A number between 1 and 500. The maximum duration for Administration Level 4.
        - maxTravelTime5 (number): A number between 1 and 200. The maximum duration for Administration Level 5.
        - averageSpeed (number): Average speed.
        - distanceUnit (str): Distance unit "km" or "mi".
        - detourFactor (number): A decimal number between 1 and 3.
        - restrictedByCountryBoundaries (boolean): A parameter taking value True or False depending on whether the country borders should be considered during the isochrone calculation.

    api_key (str): The Log-hub API key for accessing the isochrone plus service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Tuple[pd.DataFrame, pd.DataFrame]: Twio pandas DataFrames containing the original address information along 
                  with the geocoded results, and information about the reachable areas, like area name, coordinates of the area center, and population.
                  Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    mandatory_columns = {'name': 'str', 'country': 'str'}
    optional_columns = {'state': 'str', 'postalCode': 'str', 'city': 'str', 'street': 'str'}

    # Validate and convert data types
    addresses = validate_and_convert_data_types(addresses, mandatory_columns, 'mandatory', 'addresses')
    if not addresses is None:
        addresses = validate_and_convert_data_types(addresses, optional_columns, 'optional', 'addresses')

    if addresses is None:
        return None
    
    url = create_url("isochroneplus")
    
    headers = create_headers(api_key)

    payload = {
        'geocodingData': addresses.to_dict(orient='records'),
        'parameters': parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "isochrone plus")
    if response_data is None:
        return None
    else:
        geocoded_data_df = pd.DataFrame(response_data['geocodingResult'])
        reachable_areas_df = pd.DataFrame(response_data['reachableAreasTable'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return geocoded_data_df, reachable_areas_df

def forward_isochrone_plus_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'IsochronePlusSampleDataAddresses.xlsx')
    addresses_df = pd.read_excel(data_path, sheet_name='addresses', usecols='A:G').fillna("")
    parameters = {
        'level': '3',
        'maxTravelTime1': 300,
        'maxTravelTime2': 300,
        'maxTravelTime3': 100,
        'maxTravelTime4': 100,
        'maxTravelTime5': 50,
        'averageSpeed': 60,
        'distanceUnit': 'km',
        'detourFactor': 1.2, 
        'restrictedByCountryBoundaries': False
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'addresses': addresses_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}

def reverse_isochrone_plus(geocodes: pd.DataFrame, parameters: Dict, api_key: str, save_scenario = {}, show_buttons = False) -> Optional[Tuple[pd.DataFrame, pd.DataFrame]]:
    """
    Perform reverse isochrone calculation on a list of latitude and longitude coordinates.

    This function takes a DataFrame of geocodes (latitude and longitude), parameters and an API key, 
    and calculates isochrones using the Log-hub reverse isochrone service. 

    Parameters:
    geocodes (pd.DataFrame): A pandas DataFrame containing the geocodes to be reverse geocoded.
        Required columns and their types:
        - name (str): The name of the isochrone.
        - latitude (float): Latitude of the location.
        - longitude (float): Longitude of the location.

    parameters (dict): A dictionary containing parameters for isochrone calculation. 
        - level (str): A number between 1 and 5, sent as a string, and representing administration level.
        - maxTravelTime1 (number): A number between 1 and 1000. The maximum duration for Administration Level 1.
        - maxTravelTime2 (number): A number between 1 and 600. The maximum duration for Administration Level 2.
        - maxTravelTime3 (number): A number between 1 and 600. The maximum duration for Administration Level 3.
        - maxTravelTime4 (number): A number between 1 and 500. The maximum duration for Administration Level 4.
        - maxTravelTime5 (number): A number between 1 and 200. The maximum duration for Administration Level 5.
        - averageSpeed (number): Average speed.
        - distanceUnit (str): Distance unit "km" or "mi".
        - detourFactor (number): A decimal number between 1 and 3.
        - restrictedByCountryBoundaries (boolean): A parameter taking value True or False depending on whether the country borders should be considered during the isochrone calculation.

    api_key (str): The Log-hub API key for accessing the reverse isochrone plus service.

    save_scenario (dict): A dictionary containg information about saving scenario, empty by default. Allowed key vales are
                            'saveScenario' (boolean), 'overwriteScenario' (boolean), 'workspaceId' (str) and
                            'scenarioName' (str).

    show_buttons (boolean): If this parameter is set to True and the scenario is saved on the platform, the buttons linking to the output results, map, dashboard and the input table 
                           will be created. If the scenario is not saved, a proper message will be shown.

    Returns:
    Tuple[pd.DataFrame, pd.DataFrame]: Two pandas DataFrames containing the original geocode information along 
                  with the reverse geocoded address results, and information about the reachable areas, like area name, coordinates of the area center, and population.
                  Returns None if the process fails.
    """
    def create_buttons():
        links = get_workspace_entities(save_scenario, api_key)
        create_button(links = [links['map'], links['inputDataset'], links['outputDataset']], texts = ["🌍 Open Map", "📋 Show Input Dataset", "📋 Show Output Dataset"])

    geocodes_columns = {'name': 'str', 'latitude': 'float', 'longitude': 'float'}

    # Validate and convert data types
    geocodes = validate_and_convert_data_types(geocodes, geocodes_columns, 'mandatory', 'coordinates')
    if geocodes is None:
        return None
    
    url = create_url("reverseisochroneplus")
    headers = create_headers(api_key)
    payload = {
        "geocodingData": geocodes.to_dict(orient='records'),
        "parameters": parameters
    }
    payload = save_scenario_check(save_scenario, payload)

    response_data = post_method(url, payload, headers, "reverse isochrone plus")
    if response_data is None:
        return None
    else:
        geocoded_data_df = pd.DataFrame(response_data['geocodingResult'])
        reachable_areas_df = pd.DataFrame(response_data['reachableAreasTable'])
        if (show_buttons and payload['saveScenarioParameters']['saveScenario']):
            create_buttons()
        if (not payload['saveScenarioParameters']['saveScenario'] and show_buttons):
            logging.info("Please, save the scenario in order to create the buttons for opening the results on the platform.")
        return geocoded_data_df, reachable_areas_df

def reverse_isochrone_plus_sample_data():
    warnings.simplefilter("ignore", category=UserWarning)
    data_path = os.path.join(os.path.dirname(__file__), 'sample_data', 'IsochronePlusSampleDataReverse.xlsx')
    geocodes_df = pd.read_excel(data_path, sheet_name='coordinates', usecols='A:D')

    parameters = {
        'level': '3',
        'maxTravelTime1': 300,
        'maxTravelTime2': 300,
        'maxTravelTime3': 100,
        'maxTravelTime4': 100,
        'maxTravelTime5': 50,
        'averageSpeed': 60,
        'distanceUnit': 'km',
        'detourFactor': 1.2, 
        'restrictedByCountryBoundaries': False
    }
    save_scenario = {
        'saveScenario': False,
        'overwriteScenario': False,
        'workspaceId': 'Your workspace id',
        'scenarioName': 'Your scenario name'
    }
    return {'coordinates': geocodes_df, 'parameters': parameters, 'saveScenarioParameters': save_scenario}